@@ -4,8 +4,9 @@ WCAParser : turns an input string into an AST representing the
 '''
 import ply.yacc as yacc
 from wrc.parse.lexer import WCALexer
-from wrc.sema.ast import WCAGuidelines, WCARegulations, Section, Subsection,\
-                      TableOfContent, Regulation, Guideline, Article, LabelDecl
+from wrc.sema.ast import WCAGuidelines, WCARegulations, WCAStates, Section,\
+                         Subsection, TableOfContent, Regulation, Guideline,\
+                         Article, LabelDecl, StatesList, State
 from wrc.sema.check import HierarchyCheck, LabelCheck
 
 class WCAParser(object):
@@ -17,7 +18,8 @@ class WCAParser(object):
         self.doctype = WCARegulations
         self.errors = []
         self.warnings = []
-        self.sema = {WCARegulations : [HierarchyCheck],
+        self.sema = {WCAStates : [],
+                     WCARegulations : [HierarchyCheck],
                      WCAGuidelines : [HierarchyCheck, LabelCheck]}
         self.toc = None
 
@@ -76,9 +78,11 @@ class WCAParser(object):
             lhs[0].append(item)
 
     def p_content(self, content):
-        '''content : TITLE opttexts VERSION opttexts sections'''
-        content[0] = self.doctype(content[1], content[3], content[5])
-        self.toc.set_articles([a for a in content[0].sections if isinstance(a, Article)])
+        '''content : TITLE opttexts VERSION opttexts sections
+                   | TITLE STATESTAG VERSION opttexts states_sections'''
+        content[0] = self.doctype(content[1], content[3], content[4], content[5])
+        if self.toc:
+            self.toc.set_articles([a for a in content[0].sections if isinstance(a, Article)])
 
     def p_texts(self, texts):
         '''texts : textlist'''
@@ -88,7 +92,7 @@ class WCAParser(object):
 
     def p_textlist(self, textlist):
         '''textlist : textlist text
-                 | text'''
+                    | text'''
         self._act_on_list(textlist)
 
     def p_text(self, text):
@@ -100,6 +104,20 @@ class WCAParser(object):
         if len(text) > 2:
             text[0] += "\n"
 
+    def p_states_sections(self, sections):
+        '''states_sections : states_sections states_section
+                           | states_section'''
+        self._act_on_list(sections)
+
+    def p_states_section(self, body):
+        '''states_section : regularsec
+                          | states_body'''
+        body[0] = body[1]
+
+    def p_states_body(self, section):
+        '''states_body : STATESHEADER states'''
+        section[0] = StatesList(section[1], section[2])
+
     def p_sections(self, sections):
         '''sections : sections section
                     | section'''
@@ -116,14 +134,25 @@ class WCAParser(object):
         toc[0] = TableOfContent(toc[1], toc[2], [])
         self.toc = toc[0]
 
+    def p_toc_or_regular_error(self, toc):
+        '''toc : HEADERSEC error'''
+        # The p_error function just added a generic error, complete it with
+        # a more precise one
+        self.errors[-1] += ", expected text, subsections or TOC."
+
     def p_article(self, article):
         '''article : ARTICLEHEADER opttexts rules opttexts'''
         article[0] = Article(article[1][4], article[2], article[3], article[1][0],
                              article[1][1], article[1][2], article[1][3], article[1][5])
 
+    def p_article_error(self, article):
+        '''article : ARTICLEHEADER error'''
+        # The p_error function just added a generic error, complete it with
+        # a more precise one
+        self.errors[-1] += ", expected rules or optional texts."
 
     def p_regularsec(self, regularsec):
-        '''regularsec : HEADERSEC opttexts subsections'''
+        '''regularsec : HEADERSEC opttexts optsubsections'''
         texts = []
         sections = regularsec[2]
         if len(regularsec) > 3:
@@ -131,6 +160,11 @@ class WCAParser(object):
             sections = regularsec[3]
         regularsec[0] = Section(regularsec[1], texts, sections)
 
+    def p_optsubsections(self, optsubsections):
+        '''optsubsections : subsections
+                          | '''
+        optsubsections[0] = optsubsections[1] if len(optsubsections) > 1 else u""
+
     def p_opttexts(self, opttexts):
         '''opttexts : texts
                     | '''
@@ -205,7 +239,16 @@ class WCAParser(object):
             self.current_rule.append(reg)
             self.prev_indent = indentsize
 
+    def p_states(self, states):
+        '''states : states state
+                  | state'''
+        self._act_on_list(states)
+
+    def p_state(self, state):
+        '''state : STATE opttexts'''
+        state[0] = State(state[1][0], state[1][1], state[1][2], state[1][3], state[2])
+
     def p_error(self, elem):
         '''Handle syntax error'''
-        print "Syntax error on line " + str(self.lexer.lineno-1)
-        print "Unexpected " + elem.type
+        self.errors.append("Syntax error on line " + str(self.lexer.lineno)
+                           + ". Got unexpected token " + elem.type)
